#ifndef __NETWORK_H_
#define __NETWORK_H_

/*
 * network.h:
 *	Low-level network interface.
 *
 *	This interface defines a low-level network interface for sending and
 *	receiving packets between pseudo-network interfaces located on the 
 *      same or different hosts.
 */


/* get the correct version of windows nt recognised */
#define _WIN32_WINNT 0x0400
#include <windows.h>
#include <winsock.h>


#define MAX_NETWORK_PKT_SIZE	8192

#define MINIMSG_PORT 8086


typedef unsigned long network_address_t[2];

typedef void (*network_handler_t)(network_address_t, char*, int);


void 
network_address_to_sockaddr(network_address_t addr, struct sockaddr_in *sin);

void 
sockaddr_to_network_address(struct sockaddr_in *sin, network_address_t addr);

/*
 * network_initialize should be called before
 * clock interrupts start happening (or with clock interrupts disabled).
 * The initialization procedure returns 0 on success, -1 on failure.
 * The function handler(data) is called when a network packet arrives.
 */

int
network_initialize(network_handler_t handler);

/*
 * network_send_pkt returns 1 if it was able to successfully send the data.
 * Returns 0 otherwise.
 */
int
network_send_pkt(network_address_t dest_address,
		 int hdr_len, char * hdr,
		 int  data_len, char * data);


/*
 * network_my_address returns the network_address that can be used
 * to send a packet to the caller's address space.  Note that
 * an address space can send a packet to itself by specifying the result of
 * network_get_my_address() as the dest_address to network_send_pkt.
 */
void
network_get_my_address(network_address_t my_address);

int
network_translate_hostname(char* hostname, network_address_t address);

/* only used for testing; normally, you should not have to call this 
	 function. it should be called before network_initialize, and sets
	 the local UDP port to use for miniports, as well as the port number
	 to use for "remote" ports; by this mechanism, it's possible to run
	 a pair of processes on the same computer without their ports 
	 conflicting. of course, this is a hack.
*/
void
network_udp_ports(short myportnum, short otherportnum);

/* set synthetic parameters for the network, used in network_send_pkt
	 to determine whether to drop or duplicate the packet to be sent */
void
network_synthetic_params(double loss, double duplication);

#endif __NETWORK_H_

