/* Testing the reliable networking interface */


#include "minithread.h"
#include "synch.h"
#include "minimsg.h"

#include <stdio.h>
#include <stdlib.h>
#include <Time.h>
#include <string.h>


#define BUFFER_LENGTH (256*1024)


network_address_t my_address;
miniport_t myport1, myport2, myport3, myport4;
miniport_t remport1, remport2, remport3, remport4;

int check(char *buff, int len, char cmp);
long mytime();
void wait(long timeout);

const char* message1 = "Hello world";
const char* message2 = "12345678901";

char buf1[BUFFER_LENGTH];
char buf2[BUFFER_LENGTH];

double LOSSRATE = 0.15; 
double DUPLRATE = 0.1;
int FINISHED;
char* hostname;

int NRPT = 100;


/* Test 1: send "Hello World" to Thread2 through a miniport pair */
void send1(int nested) {
	int len;
	
	if (!nested) {	
		myport1 = miniport_local_create();
		myport2 = miniport_local_create();
		remport1 = miniport_remote_create(my_address, 0); 
		remport2 = miniport_remote_create(my_address, 1); 
		printf("TESTER:\tTest 1\n");
	}
	
	strcpy(buf1, message1);
	printf("TESTER: 1st send - sending %d bytes: %s\n", strlen(buf1)+1, buf1);
	
	len = minimsg_send(myport1, remport1, buf1, 12);
	printf("TESTER: minimsg_send sent %d bytes\n", len);
}

/* Test 2: receiver sends back received message through another miniport pair */
void send2(int nested) {
	int len;
	
	if (!nested) {
		myport1 = miniport_local_create();
		myport2 = miniport_local_create();
		remport1 = miniport_remote_create(my_address, 0); 
		remport2 = miniport_remote_create(my_address, 1); 
		printf("TESTER:\tTest 2\n");
	}

	strcpy(buf1, message1);
	
	len = minimsg_send(myport2, remport2, buf1, 12);
	printf("TESTER: minimsg_send sent %d bytes\n", len);

	len = 10000;
	len = minimsg_receive(myport2, &remport2, buf1, &len);
	
	printf("TESTER: minimsg_recv returned %d bytes\n", len);
	printf("TESTER: Received from receiver: %s\n", buf1);
}

/* Test 3 - tests 1 and 2 repeated several times */
void send3(void) {
	int i;

	myport1 = miniport_local_create();
	myport2 = miniport_local_create();
	remport1 = miniport_remote_create(my_address, 0); 
	remport2 = miniport_remote_create(my_address, 1); 

	printf("TESTER:\tTest 3\n");
	for(i = 1; i <= NRPT; i++){
		printf("TESTER: iteration %d.\n", i);
		send1(1);
		send2(1);
	}

	printf("TESTER: Test 3 done\n");
}

int send4thread(int *arg){
	int len;
	int i;
	
	for(i = 1; i <= NRPT; i++){

		/* Test 1 */

		strcpy(buf2, message2);

		printf("TESTER: (send4thread)[%d] Sending %s to receiver\n", i, buf2);
		len = minimsg_send(myport3, remport3, buf2, 12);
		printf("TESTER:(send4thread)[%d] minimsg_send sent %d bytes\n", i, len);

		/* Test 2 */

		printf("TESTER: (send4thread)[%d] Sending %s to receiver\n", i, buf2);
		len = minimsg_send(myport4, remport4, buf2, 12);
		printf("TESTER:(send4thread)[%d] minimsg_send sent %d bytes\n", i, len);
		len = 10000;
		len = minimsg_receive(myport4, &remport4, buf2, &len);
		printf("TESTER:(send4thread)[%d] minimsg_recv: %d bytes returned\n", i, len);
		printf("TESTER:(send4thread)[%d] Received from receiver: %s\n", i, buf2);
	}

	FINISHED=1;

	return 0;
}

/* Test 4: sender forks off Thread1b and receiver forks off Thread2b.
	 Thread1-Thread2 and Thread1b-Thread2b pairs each repeat Test 3. - Will
	 give you output for each iteration, with iteration number in square
	 brackets [] after thread name. */
void send4(int nested) {
	int i;

	myport1 = miniport_local_create();
	myport2 = miniport_local_create();
	myport3 = miniport_local_create();
	myport4 = miniport_local_create();
	remport1 = miniport_remote_create(my_address, 0); 
	remport2 = miniport_remote_create(my_address, 1); 
	remport3 = miniport_remote_create(my_address, 2); 
	remport4 = miniport_remote_create(my_address, 3); 

	if (!nested)
		printf("TESTER:\tTest 4\n");
		
	FINISHED=0;
		
	printf("\nTESTER: forking off send4thread\n");
	minithread_fork(send4thread, NULL);

	for(i = 1; i <= NRPT; i++){
		printf("TESTER: Main thread is sending message %d.\n", i);
		send1(1);
		send2(1);
	}
		
	while(!FINISHED)
		;
	
	if (!nested)
		printf("TESTER: Fininished test 4.\n");
}

void send5(void) {
	printf("TESTER:\tTest 5: LOSSRATE=%lf DUPLRATE=%lf\n", LOSSRATE, DUPLRATE);
	network_synthetic_params(LOSSRATE, DUPLRATE);
	send4(1);

  printf("TESTER: Finished test 5.\n");
}	

void send6(void) {
	int len;
	int i;	

	myport1 = miniport_local_create();
	myport2 = miniport_local_create();
	remport1 = miniport_remote_create(my_address, 0); 
	remport2 = miniport_remote_create(my_address, 1); 

	printf("TESTER:\tTest 6\n");

	for(i = 0; i <= 999; i++)
		buf1[i] = 'a';
	for(i = 1000; i <= 1003; i++)
		buf1[i] = 'b';

	len = 1004;
	len = minimsg_send(myport1, remport1, buf1, len);
	printf("TESTER: minimsg_send sent %d bytes\n", len);
}

void send7(void) {
	int len;
	int i;

	myport1 = miniport_local_create();
	myport2 = miniport_local_create();
	remport1 = miniport_remote_create(my_address, 0); 
	remport2 = miniport_remote_create(my_address, 1); 

	printf("\n\n TESTER:\tTest 7\n");
	fflush(stdout);

	for(i = 0; i <= 999; i++)
		buf1[i] = 'c';

	len = 1000;
	len = minimsg_send(myport1, remport1, buf1, len);
	printf("TESTER: minimsg_send sent %d bytes\n", len);
}

void send8(void) {
	int len;
	int i;	

	myport1 = miniport_local_create();
	myport2 = miniport_local_create();
	remport1 = miniport_remote_create(my_address, 0); 
	remport2 = miniport_remote_create(my_address, 1); 

	network_synthetic_params(LOSSRATE, DUPLRATE);

	printf("TESTER:\tTest 8\n");

	for(i = 0; i <= 99999; i++)
		buf1[i] = 'd';

	len = 100000;
	len = minimsg_send(myport1, remport1, buf1, len);
	printf("TESTER: minimsg_send sent %d bytes\n", len);
}

int send9thread(int* arg) {
	miniport_t remport = miniport_remote_create(my_address, 0);
	miniport_t myport = (miniport_t) arg;
	int i;
	int len;

	for(i = 0; i <= 99999; i++)
		buf2[i] = 'f';

	len = 100000;
	len = minimsg_send(myport, remport, buf2, len);
	printf("TESTER:(send9thread) minimsg_send sent %d bytes\n", len);

	wait(20000);

	FINISHED=1;

	return 0;

}

void send9(void) {	
	int len;
	int i;
	
	myport1 = miniport_local_create();
	myport2 = miniport_local_create();
	remport1 = miniport_remote_create(my_address, 0); 
	remport2 = miniport_remote_create(my_address, 1); 

	network_synthetic_params(LOSSRATE, DUPLRATE);

	FINISHED=0;

	printf("TESTER:\tTest 9\n");
	printf("TESTER:(send9) Forking send9thread\n");
	minithread_fork(send9thread, (int *)myport1);

	for(i = 0; i <= 99999; i++)
		buf1[i] = 'e';

	len = 100000;
	len = minimsg_send(myport1, remport1, buf1, len);
	printf("TESTER:(send9) minimsg_send sent %d bytes\n", len);

	wait(2000);

	while(!FINISHED)
		;
}

void recv1(int nested) {
	int len;

	if (!nested) {
		myport1 = miniport_local_create();
		myport2 = miniport_local_create();
		remport1 = miniport_remote_create(my_address, 0); 
		remport2 = miniport_remote_create(my_address, 1); 
		printf("TESTER: \t Test 1\n");
	}

	len = 10000;
	len = minimsg_receive(myport1, &remport1, buf1, &len);
	printf("TESTER: minimsg_recv: %d bytes returned\n", len);
	printf("TESTER: Received from sender: %s\n", buf1);
}

void recv2(int nested) {
	int len;
	
	
	if (!nested) {
		myport1 = miniport_local_create();
		myport2 = miniport_local_create();
		remport1 = miniport_remote_create(my_address, 0); 
		remport2 = miniport_remote_create(my_address, 1); 
		printf("TESTER: \t Test 2\n");
	}

	len = 10000;
	len = minimsg_receive(myport2, &remport2, buf1, &len);

	printf("TESTER: Sending back %s to sender\n", buf1);
	fflush(stdout);

	len = minimsg_send(myport2, remport2, buf1, len);

	printf("TESTER: minimsg_send sent %d bytes\n", len);
}

void recv3(void) {
	int len;
	int i;

	myport1 = miniport_local_create();
	myport2 = miniport_local_create();
	remport1 = miniport_remote_create(my_address, 0); 
	remport2 = miniport_remote_create(my_address, 1); 
	
	printf("TESTER: \t Test 3\n");

	for(i = 1; i <= NRPT; i++){

		/* Test 1 */
		len = 25000;
		if((len = minimsg_receive(myport1, &remport1, buf1, &len)) != 12
			|| strcmp(buf1, message1) != 0){
			printf("TESTER: minimsg_recv return value error : %d bytes returned\n", len);
			printf("TESTER: Received from sender: %s\n", buf1);
		}
		else
			printf("TESTER: Received packet %d from test 1.\n", i);

		/* Test 2 */
		len = 25000;
		if((len = minimsg_receive(myport2, &remport2, buf1, &len)) != 12
			|| strcmp(buf1, message1) != 0){
			printf("TESTER: minimsg_recv return value error : %d bytes returned\n", len);
			printf("TESTER: Received from sender: %s\n", buf1);
		}
		else
			printf("TESTER: Received packet %d from test 2.\n", i);

		if((len = minimsg_send(myport2, remport2, buf1, len)) != 12)
			printf("TESTER: could not send, minimsg_send sent %d bytes\n", len);
		else
			printf("TESTER: Sent packet %d for test 2.\n", i);

	}

	printf("TESTER: Test 3 done\n");
}

int recv4thread(int *arg){
	int len;
	int i;

	for(i = 1; i <= NRPT; i++){
		/* Test 1 */
		len = 25000;
		len = minimsg_receive(myport3, &remport3, buf2, &len);
		printf("TESTER:(recv4thread)[%d] minimsg_recv for test 1: %d bytes returned\n", i, len);
		printf("TESTER:(recv4thread)[%d] Received from sender: %s\n", i, buf2);
		
		/* Test 2 */
		len = 25000;
		len = minimsg_receive(myport4, &remport4, buf2, &len);
		printf("TESTER:(recv4thread)[%d] minimsg_recv for test 2: %d bytes returned\n", i, len);
		printf("TESTER:(recv4thread)[%d] Received from sender: %s\n", i, buf2);

		printf("TESTER: (recv4thread)[%d] Sending %s to sender\n", i, buf2);
		len = minimsg_send(myport4, remport4, buf2, len);
		printf("TESTER:(recv4thread)[%d] minimsg_send sent %d bytes\n", i, len);
	}

	FINISHED=1;

	return 0;
}

void recv4(int nested) {
	int i;

	myport1 = miniport_local_create();
	myport2 = miniport_local_create();
	myport3 = miniport_local_create();
	myport4 = miniport_local_create();
	remport1 = miniport_remote_create(my_address, 0); 
	remport2 = miniport_remote_create(my_address, 1); 
	remport3 = miniport_remote_create(my_address, 2); 
	remport4 = miniport_remote_create(my_address, 3); 
	
	if (!nested)
		printf("\n TESTER:\tTest 4\n");
	
	FINISHED=0;
			
	printf("TESTER: forking off recv4thread\n");
	minithread_fork(recv4thread, NULL);

	for(i = 1; i <= NRPT; i++){
		printf("TESTER: iteration %d.\n", i);
		recv1(1);
		recv2(1);
	}

	while(!FINISHED)
		;
}

void recv5(void) {
	printf("TESTER:\tTest 5: LOSSRATE=%lf, DUPLRATE=%lf\n", LOSSRATE, DUPLRATE);
	network_synthetic_params(LOSSRATE, DUPLRATE);
	recv4(1);
}

void recv6(void) {
	int len;
	
	myport1 = miniport_local_create();
	myport2 = miniport_local_create();
	remport1 = miniport_remote_create(my_address, 0); 
	remport2 = miniport_remote_create(my_address, 1); 
	
	printf("TESTER:\tTest 6\n");
	
	len = 1000;
	len = minimsg_receive(myport1, &remport1, buf1, &len); 
	printf("TESTER: minimsg_recv - %d bytes returned\n", len);
	if(check(buf1, 1000, 'a') != 0)
		printf("TESTER: Bogus message received %c%c...\n", buf1[0], buf1[1]);
		
	fflush(stdout);
	len = 4;
	len = minimsg_receive(myport1, &remport1, buf1, &len);
	printf("TESTER: minimsg_recv - %d bytes returned\n", len);
	if(check(buf1, 4, 'b') != 0)
		printf("TESTER: Bogus message received %c%c...\n", buf1[0], buf1[1]);
	fflush(stdout);
}

void recv7(void) {
	int len;
	
	myport1 = miniport_local_create();
	myport2 = miniport_local_create();
	remport1 = miniport_remote_create(my_address, 0); 
	remport2 = miniport_remote_create(my_address, 1); 
	
	printf("TESTER : \t Test 7\n");

	len = 5000;
	len = minimsg_receive(myport1, &remport1, buf1, &len);
	printf("TESTER: minimsg_recv - %d bytes returned\n", len);
	if(check(buf1, 1000, 'c') != 0)
		printf("TESTER: Bogus message received %c%c...\n", buf1[0], buf1[1]);
	fflush(stdout);
}

void recv8(void) {
	int len;
	int starttime;
	int sofar;

	myport1 = miniport_local_create();
	myport2 = miniport_local_create();
	remport1 = miniport_remote_create(my_address, 0); 
	remport2 = miniport_remote_create(my_address, 1); 
	
	printf("TESTER:\tTest 8\n");
	starttime = mytime();
	sofar = 0;
	while(sofar < 100000){
		len = 210000-sofar;
		len = minimsg_receive(myport1, &remport1, buf1, &len);
		printf("(%d)", len);
		if(len > 0 && check(buf1, len, 'd') == 0)
			sofar += len;
		else printf("\n TESTER:Bogus message %c%c...", buf1[0], buf1[1]);
		printf(".%d.", sofar);
	}

	printf("\n TESTER:Received %d bytes\n", sofar);
}

void recv9(void) {
	int len;
	int starttime;
	int sofar;
	
	myport1 = miniport_local_create();
	myport2 = miniport_local_create();
	remport1 = miniport_remote_create(my_address, 0); 
	remport2 = miniport_remote_create(my_address, 1); 
	
	printf("TESTER:\tTest 9\n");
	starttime = mytime();
	sofar = 0;
	while(sofar < 200000){
		len = 210000-sofar;
		len = minimsg_receive(myport1, &remport1, buf1, &len);
		printf("(%d)", len);
		if(len > 0 && check(buf1, len, 'e') == 0 || check(buf1, len, 'f') == 0 )
			sofar += len;
		else printf("\n TESTER:Bogus message %c%c...\n", buf1[0], buf1[1]);
		printf(".%d.", sofar);
	}

	printf("\n TESTER:Received %d bytes\n", sofar);
}

int check(char *buff, int len, char cmp){

	int i;

	for(i = 0; i <= len-1; i++)
		if(buff[i] != cmp)
			return -1;

	return 0;
}

// return relative time in millisecs
long mytime(){

	return ((long) clock());

}

// wait for timeout millisecs
void wait(long timeout){

	long now = mytime();

	while(now+timeout > mytime())
		;
}

int sender(int* arg) {
	char** argv = (char **) arg;
	
	if (network_translate_hostname(hostname, my_address) < 0) {
		printf("Could not resolve hostname, exiting.\n");
		exit(-1);
	}

	switch (argv[1][0]) {
	case '1':
		send1(0);
		break;
	case '2':
		send2(0);
		break;
  case '3':
		send3();
		break;
	case '4':
		send4(0);
		break;
	case '5':
		send5();
		break;
	case '6':
		send6();
		break;
	case '7':
		send7();
		break;
	case '8':
		send8();
		break;
	case '9':
		send9();
		break;
	}

	return 0;
}

int receiver(int* arg) {
	char** argv = (char **) arg;
	
	if (network_translate_hostname(hostname, my_address) < 0) {
		printf("Could not resolve hostname, exiting.\n");
		exit(-1);
	}

	if (argv == NULL)
		recv1(0);
	else {
		switch (argv[1][0]) {
		case '1':
			recv1(0);
			break;
		case '2':
			recv2(0);
			break;
		case '3':
			recv3();
			break;
		case '4':
			recv4(0);
			break;
		case '5':
			recv5();
			break;
		case '6':
			recv6();
			break;
		case '7':
			recv7();
			break;
		case '8':
			recv8();
			break;
		case '9':
			recv9();
			break;
		}
	}

	return 0;
}

main(int argc, char** argv) {
	printf("TESTER: Testing the minimsg interface\n\n");

	srand(20);

	if (argc > 2) {
    hostname = argv[2];
		network_udp_ports(8088, 8086);	
		minithread_system_initialize(sender, (int *) argv);
  }
  else {
		network_udp_ports(8086, 8088);
		if (argc == 1)
			minithread_system_initialize(receiver, NULL);
		else
			minithread_system_initialize(receiver, (int *) argv);
  }
}
